// Qarawiyyn Mosque Pages 104 - 115 of Islamic Geometric Patters
// Thames & Hudson by Eric Broug
// ISBN 978-0-500-28721-7
// ISBN 10: 050028721XISBN 13: 9780500287217
// code by Rupert Russell February 9 March 2022 v061
// https://rupert.id.au/QarawiyyinMosque/index.php

// reference sites
// https://x-engineer.org/linear-interpolation-extrapolation-calculator/#formula
// https://rosettacode.org/wiki/Line_circle_intersection
// Thanks to REAS for line circle intesection code
// https://openprocessing.org/sketch/8009#
// by https://openprocessing.org/user/54?view=sketches
// code on github at: https://github.com/rupertrussell/QarawiyynMosque

boolean drawGuideLines = true;
boolean drawDesignLines = false;

boolean drawGuidePoints = false;
boolean numberGuidePoints = false;

boolean numberPoints = false;
boolean designPoints = false;

boolean numberDesignPoints = false;

boolean drawEllipses = false;

boolean InnerCircleDots = false;
boolean InnerCircleText = false;

boolean InnerMostCircleDots = false;
boolean InnerMostCircleText = false;
boolean intersect = false;

int testEllipseSize = 10;

float scale = 900;
int counter = 0;
int designWeight = 2;
float guideWeight = 1;

int srrr = 0;  // red value of design Stroke
int sggg = 0;  // green value of design Stroke
int sbbb = 0;  // blue value of design Stroke

int grrr = 0;  // red value of guide Stroke
int gggg = 0;  // green value of guide Stroke
int gbbb = 0;  // blue value of guide Stroke

float[] xOuterCircle = new float[25]; // store points on outer circle
float[] yOuterCircle = new float[25]; // store points on outer circle

float[] xSquaresToOuterCircle = new float[49]; // store points on outer circle from extended square lines used in step 08
float[] ySquaresToOuterCircle = new float[49]; // store points on outer circle from extended square lines used in step 08

float[] xLinesToOuterCircle = new float[90]; // store points on outer circle from extended parallel lines used in step 09
float[] yLinesToOuterCircle = new float[90]; // store points on outer circle from extended parallel lines used in step 09

float[] xLinesToStep11Circles = new float[80]; // store points used in step 11
float[] yLinesToStep11Circles = new float[80]; // store points used in step 11

float[] xLinesToStep12Circles = new float[14]; // store points used in step 12
float[] yLinesToStep12Circles = new float[14]; // store points used in step 12

float[] xLinesToStep13Circles = new float[55]; // store points used in step 13
float[] yLinesToStep13Circles = new float[55]; // store points used in step 13

float[] xDesignLinesInStep13 = new float[25]; // store design points used in step 13
float[] yDesignLinesInStep13 = new float[25]; // store design points used in step 13

float[] xDesignLinesInStep14 = new float[36]; // store design points used in step 14
float[] yDesignLinesInStep14 = new float[36]; // store design points used in step 14

float[] xDesignLinesInStep15 = new float[36]; // store design points used in step 15
float[] yDesignLinesInStep15 = new float[36]; // store design points used in step 15

float[] xDesignLinesInStep16 = new float[32]; // store design points used in step 16
float[] yDesignLinesInStep16 = new float[32]; // store design points used in step 16

float[] xDesignLinesInStep17 = new float[49]; // store design points used in step 17
float[] yDesignLinesInStep17 = new float[49]; // store design points used in step 17

float[] xDesignLinesInStep18 = new float[36]; // store design points used in step 18
float[] yDesignLinesInStep18 = new float[36]; // store design points used in step 18

int Step13Counter = 0; // track which array cell to save into
int Step14Counter = 0; // track which array cell to save into
int Step15Counter = 0; // track which array cell to save into
int Step17Counter = 0; // track which array cell to save into
int Step18Counter = 0; // track which array cell to save into

float innerRaduis = 0; // radius of inner circle calculated in  step06
float[] xInnerCircle = new float[13]; // store points on inner circle
float[] yInnerCircle = new float[13]; // store points on inner circle

float innerMostRaduis = 0; // radius of inner circle calculated in  step07
float[] xInnerMostCircle = new float[13]; // store points on innter most circle
float[] yInnerMostCircle = new float[13]; // store points on innter most circle

float[] xOuterHexagon = new float[25]; // store points on outer hexagon in step08
float[] yOuterHexagon = new float[25]; // store points on outer hexagon in step08

float[] xDesign = new float[99]; // store for points of final design
float[] yDesign = new float[99]; // store for points of final design

float[] xTemp = new float[99]; // Used for testing
float[] yTemp = new float[99]; // Used for testing

float xCenter = scale / 2;
float yCenter = scale / 2;

float m = 0; // used to calculate extrapolation points


float x, y; // used to hold intersection point
float a1;
float b1;
float c1;
float a2;
float b2;
float c2;
float det;

int xOffset = 5; // text offset for point lables
int yOffset = 5; // text offset for point lables

void setup() {
  
  // size(scale,scale)
  size(1000, 1000);
  smooth();

  background(255);
  noLoop();
}

void draw() {
  translate(50, 50);
  step01(); // Outer circle
  step06(); // Inner circle
  step07(); // Inner Most circle
  step08(); // Extend the stide of the squares in step06 & step07 to the outer hexagon
  step09();
  step11();
  step12();
  step13();
  step14();
  step15();
  step16();
  step17();
  step18();
 // step19();


  designPoints(0, 0, 0, numberDesignPoints);  // text colour for design points
  guidePoints(0, 0, 0, numberPoints, true);  //  text colour for guide points

  save("Qarawiyyn_Mosque_128.png");
}

float extrapolateY(float x1, float y1, float x2, float y2, float x) {
  m = (y2 - y1) / (x2 - x1);
  return y1 + m * (x - x1);
}


float extrapolateX(float x1, float y1, float x2, float y2, float y) {
  m = (y2 - y1) / (x2 - x1);
  return x1 + ((y - y1) * (x2 - x1)) / (y2 - y1);
}
